package lectura_ficheros;


import global.GlobalData;

import java.util.StringTokenizer;

/**
 * Maneja el contenido de cualquier fichero de datos teniendo en cuenta que dispondremos
 * de metadatos, de la tabla con los lugares y los datos
 * y de una variable que indica si la primera fila del fichero es un total de los datos
 * que aparecen en las siguientes lineas
 * */
public class ContenidoFichero {

	String[] metadatos;		//con el formato indicado por GlobalData.Metadatos[]
	Tabla tabla;			//tabla con el formato: LUGAR	DATO	DATO	DATO...
	boolean primeraLineaTotal;
	
	/**
	 * Constructor
	 * */
	ContenidoFichero (){
		metadatos = new String[GlobalData.NUMMETADATOS];
		tabla =  new Tabla(GlobalData.NUMFILAS, GlobalData.NUMCOLS);
		primeraLineaTotal = false;
	}
	
	/**
	 * Constructor por parmetros
	 * */
	ContenidoFichero (String [] ametadatos, Tabla atabla){
		metadatos = ametadatos;
		tabla =  atabla;
		primeraLineaTotal = false;
	}
	
	/**
	 * Asigna a <i>metadatos</i> el valor de <i>ametadatos</i>
	 * @param ametadatos Valor a asignar
	 * */
	public void setMetadatos(String[] ametadatos) {
		metadatos = ametadatos;		
	}

	/**
	 * Asigna a <i>tabla</i> el valor de <i>datos</i>
	 * @param datos Valor a asignar
	 * */
	public void setDatos(Tabla datos) {
		tabla = datos;
		
	}
	
	/**
	 * Asigna a <i>primeraLineaTotal</i> el valor de <i>bool</i>
	 * @param bool Valor a asignar
	 * */
	public void setPrimeraLineaTotal(boolean bool){
		primeraLineaTotal = bool;
	}

	/**
	 * @return Devuelve el valor de <i>metadatos</i>
	 * */
	public String[] getMetadatos() {
		return metadatos;
	}

	/**
	 * @return Devuelve el valor de <i>tabla</i>
	 * */
	public Tabla getDatos() {
		return tabla;
	}
	
	/**
	 * @return Devuelve el valor de <i>primeraLineaTotal</i>
	 * */
	public boolean getPrimeraLineaTotal(){
		return primeraLineaTotal;
	}

	/**
	 * Devuelve el valor de la tabla en la fila y columna dadas
	 * @param fila Numero de fila del datos a buscar
	 * @param col Numero de columna del datos a buscar
	 * @return Devuelve el valor de <i>tabla</i> en <i>fila</i> <i>col</i>
	 * */
	public String getDataAt(int fila, int col){
		return tabla.tabla[fila][col];
	}

	/**
	 * Inserta una fila con datos en <i>tabla</i>
	 * @param st <b>StringTokenizer</b> con los datos de la fila a introducir
	 * */
	public void insertaFila(StringTokenizer st) {
		tabla.insertaFila(st);
	}
	
	/**
	 * Devuelve el numero de filas de <i>tabla</i>
	 * @return numero de filas de <i>tabla</i>
	 * */
	public int countTuplas(){
		return tabla.countFilas();
	}
	
	/**
	 * Muestra el vector <i>metadatos</i> en la salida estandar
	 * */
	public void muestraMetadatos(){
		for (int i=0; i<metadatos.length; i++){
			System.out.println(metadatos[i]);
		}
	}
	
	/**
	 * Muestra la matriz <i>tabla</i> en la salida estandar
	 * */
	public void muestraDatos(){
		for (int i=0; i<tabla.countFilas(); i++){
			for (int j=0; j<tabla.countCols(); j++){
				System.out.print(tabla.tabla[i][j] + "\t");
			}
			System.out.println();
		}
	}
	
	/**
	 * Muestra el vector <i>metadatos</i> y la matriz <i>tabla</i> en la salida estandar
	 * */
	public void muestraContenido(){
		muestraMetadatos();
		muestraDatos();
	}
}
