package lectura_ficheros;

/*
 * Copyright (c) 1995 - 2008 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Sun Microsystems nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */ 

import java.io.*;
import java.util.StringTokenizer;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Crea una ventana para la seleccion de archivos e introduccion de datos relativos
 * al contenido de estos para procesar y guardar en una ED los datos contenidos en ellos
 * */
public class Explorador extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	
	
    JButton openButton, readButton;
    JTextFieldTabulador separ;
    JTextField num_lin_oe, num_lin_uni, num_lin_fec, num_lin_datos, log; 
    Checkbox primeraLinea;
    Choice aditividadT, aditividadE, frecuencia, tipoDato, naturalezaDato, nivelJerarquia;
    JTextField sesionID;
    
    JFileChooser fc;
    File file;
    
    ContenidoFichero contenido;// = new ContenidoFichero();


    /**
     * Constructor
     * Dispone todos los elementos que se veran en la ventana generada
     * */
    public Explorador() {
        super(new BorderLayout());
        contenido = new ContenidoFichero();
        
        //crear campos JTextFields para la entrada de datos
        separ = new JTextFieldTabulador(1);
        //Desabilitamos la funcin del tabulador como tecla especial
        separ.setFocusTraversalKeysEnabled(false);     

        num_lin_oe = new JTextField(3);
        num_lin_uni = new JTextField(3);
        num_lin_fec = new JTextField(3);
        num_lin_datos = new JTextField(3);
        //creamos el JTextField para el archivo seleccionado
        log = new JTextField(30);
        
        //crear etiquetas
        JLabel lsepar, loe, luni, lfec, ldatos, laditT, laditE, lfrec, ltipo, lnatur, lreg, lsesion;
        lsepar = new JLabel("Separador");
        loe = new JLabel("Linea Obj. Est");
        luni = new JLabel("Linea Uds");
        lfec = new JLabel("Linea Fechas");
        ldatos = new JLabel("Linea Datos");
        laditT = new JLabel("Aditividad Temporal");
        laditE = new JLabel("Aditividad Espacial");
        lfrec = new JLabel("Frecuencia");
        ltipo = new JLabel("Tipo Dato");
        lnatur = new JLabel("Naturaleza Dato");
        lreg = new JLabel("Region");
        lsesion = new JLabel("Sesion ID");
        
        //Creamos el checkbox que indicar si la primera lnea corresponde a un nivel
        //de jerarquia superior
        primeraLinea = new Checkbox("La primera linea corresponde a la suma de los datos",true);
        
        //Ponemos en un panel las etiquetas y los JTextFields
        JPanel textAreasPanel = new JPanel(); //use FlowLayout
        textAreasPanel.add(lsepar);			textAreasPanel.add(separ);
        textAreasPanel.add(loe);			textAreasPanel.add(num_lin_oe);
        textAreasPanel.add(luni);	        textAreasPanel.add(num_lin_uni);
        textAreasPanel.add(lfec);	        textAreasPanel.add(num_lin_fec);
        textAreasPanel.add(ldatos);	        textAreasPanel.add(num_lin_datos);
        textAreasPanel.add(primeraLinea);
        
        //Creamos la lista desplegable de aditividadT
        aditividadT = new Choice();
        aditividadT.add("Aditivo");
        aditividadT.add("Promedio");
        aditividadT.add("Maximo");
        aditividadT.add("Minimo");
        
        //Creamos la lista desplegable de aditividadE
        aditividadE = new Choice();
        aditividadE.add("Aditivo");
        aditividadE.add("Promedio");
        aditividadE.add("Maximo");
        aditividadE.add("Minimo");
        
        //Creamos la lista desplegable de Frecuencia
        frecuencia = new Choice();
        frecuencia.add("Dia");
        frecuencia.add("Semana");
        frecuencia.add("Quincena");
        frecuencia.add("Mes");
        frecuencia.add("Ano");
        
        //Creamos la lista desplegable de tipoDato
        tipoDato = new Choice();
        tipoDato.add("INT");
        tipoDato.add("FLOAT");
        tipoDato.add("STRING");
        tipoDato.add("BOOL");
        
        //Creamos la lista desplegable de naturalezaDato
        naturalezaDato = new Choice();
        //TODO leer de la BD
        //TODO habr que hacer un botn: agregar nueva variable
        //con un nuevo frame que capture los datos necesarios!!
        naturalezaDato.add("Demografia");
        naturalezaDato.add("Medio Ambiente");
        
        //Creamos la lista desplegable de nivelJerarquia
        nivelJerarquia = new Choice();
        //TODO leer de la BD
        nivelJerarquia.add("Pais");
        nivelJerarquia.add("Autonomia");
        nivelJerarquia.add("Privincia");
        nivelJerarquia.add("Municipio");
        
        sesionID = new JTextField(20);
        sesionID.setText("INE");
        
        //Ponemos en un panel las etiquetas y los JTextFields
        JPanel listaDespPanel = new JPanel(); //use FlowLayout
        listaDespPanel.add(laditT);			listaDespPanel.add(aditividadT);
        listaDespPanel.add(laditE);			listaDespPanel.add(aditividadE);
        listaDespPanel.add(lfrec);			listaDespPanel.add(frecuencia);
        listaDespPanel.add(ltipo);			listaDespPanel.add(tipoDato);
        listaDespPanel.add(lnatur);			listaDespPanel.add(naturalezaDato);
        listaDespPanel.add(lreg);			listaDespPanel.add(nivelJerarquia);
        
        //creamos el boton para abrir el explorador y buscar archivos
        openButton = new JButton("Seleccionar Archivo...");
        openButton.addActionListener(this);
        //y lo anhadimos a un panel
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(openButton);
        
        //creamos el boton para leer el archivo seleccionado
        readButton = new JButton("Leer Archivo");
        readButton.addActionListener(this);
        //y lo anhadimos a un panel junto con "log"
        JPanel readPanel = new JPanel(); 
        readPanel.add(lsesion);		readPanel.add(sesionID);
        readPanel.add(log);
        readPanel.add(readButton);
        

        
        JPanel norte = new JPanel();
        norte.add(buttonPanel);
        norte.add(textAreasPanel);
        add(norte, BorderLayout.NORTH);
        add(listaDespPanel, BorderLayout.CENTER);
        add(readPanel, BorderLayout.SOUTH);
        
        //Creamos un explorador
        fc = new JFileChooser();
        
    }

	/**
	 * Controla la accion dependiendo del Evento generado
	 * @param e Objeto <b>ActionEvent</b> con el evento generado
	 * */
    public void actionPerformed(ActionEvent e) {

        //Acciones del boton openButton
        if (e.getSource() == openButton) {
            int returnVal = fc.showOpenDialog(Explorador.this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file = fc.getSelectedFile();
                //This is where a real application would open the file.
                log.setText(file.getAbsolutePath());
            } else {
            	log.setText("Open command cancelled by user.");
            }

        //Acciones del boton readButton
        } else if (e.getSource() == readButton) {
            char carsep = separ.getText().charAt(0);
            FicheroDatos fd = new FicheroDatos(file, carsep);
            int num_l_ud = 0;
            
            try {
            	if (num_lin_uni.getText() != ""){
            		num_l_ud = Integer.parseInt(num_lin_uni.getText());
            	}
            	fd.leerFicheroDatos(log.getText(), carsep, 
						Integer.parseInt(num_lin_oe.getText()), num_l_ud, 
						Integer.parseInt(num_lin_fec.getText()), Integer.parseInt(num_lin_datos.getText()));

            	String[] varUdsFecha = fd.getVarUdsFechaFTabular();
            	String fechasOK = tratarFechas(varUdsFecha[2], frecuencia.getSelectedItem ());
				String metadatos[] = {varUdsFecha[0], aditividadT.getSelectedItem (), aditividadE.getSelectedItem (), frecuencia.getSelectedItem (), fechasOK, 
						varUdsFecha[1], nivelJerarquia.getSelectedItem (), tipoDato.getSelectedItem (), naturalezaDato.getSelectedItem (), sesionID.getText ()};
            	
				contenido.setMetadatos(metadatos);
				contenido.setDatos(fd.getTabla());
				if (primeraLinea.getState() == true){
					contenido.setPrimeraLineaTotal(true);
				}else{
					contenido.setPrimeraLineaTotal(false);
				}
				//Abrimos ventana nueva para comprobar y corregir los datos
				PanelOpciones po= new PanelOpciones("Modificacion de datos", file, contenido);

			} catch (NumberFormatException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
        }
    }

    /**
     * Trata una cadena <b>String</b> en busca de fechas contenidas en ella
     * @param lineaConFechas <b>String</b> con la linea sin tratar que contiene las fechas
     * @param frecuencia Puede ser 'dia','semana','quincena','mes','ao' -- segun indica el diseo de la BD de GoodWave
     * */
    private String tratarFechas(String lineaConFechas, String frecuencia) {
    	String fechasTratadas = "";
    	StringTokenizer st = new StringTokenizer(lineaConFechas, ";");
    	int numTokens = st.countTokens(); 
    	
    	//TODO hay que ver que fechas vamos a considerar
    	if (frecuencia.compareToIgnoreCase("dia")==0 || frecuencia.compareToIgnoreCase("semana")==0 || frecuencia.compareToIgnoreCase("quincena")==0){
//    		for (int i=0; i<numTokens;i++){
//    			String lin = st.nextToken();
//    			fechasTratadas = fechasTratadas + buscaDia(lin) + "-" + buscaMes(lin) + "-" + buscaAno(lin);
//    			if (i!=numTokens-1){
//    				fechasTratadas = fechasTratadas + ";";
//    			}
//    		}
    		//TODO 
    	}else if (frecuencia.compareToIgnoreCase("mes")==0){
//    		for (int i=0; i<numTokens;i++){
//    			String lin = st.nextToken();
//    			fechasTratadas = fechasTratadas + buscaMes(lin) + "-" + buscaAno(lin);
//    			if (i!=numTokens-1){
//    				fechasTratadas = fechasTratadas + ";";
//    			}
//    		}
    	}else if (frecuencia.compareToIgnoreCase("ano")==0){
    		for (int i=0; i<numTokens;i++){
    			fechasTratadas = fechasTratadas + buscaAno(st.nextToken());
    			if (i!=numTokens-1){
    				fechasTratadas = fechasTratadas + ";";
    			}
    		}
    	}
		
		return fechasTratadas;
	}

//    private String buscaDia(String lin) {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	private String buscaMes(String lin) {
//		// TODO Auto-generated method stub
//		return null;
//	}

	/**
     * Busca una cadena del tipo 1998, es decir, 4 dgitos consecutivos y aislados.
     * @param linea <b>String</b> con la linea sin tratar con la fecha de 4 digitos
     * @return Objeto <b>String</b> con la fecha hallada del tipo aaaa. P.e. 2003
     * */
	private String buscaAno(String linea) {
		char[] ano = new char[4];
		int cont = 0;
		boolean encontrado = false;
		
		for (int i=0; i<linea.length() && !encontrado;i++){
			if (Character.isDigit(linea.charAt(i))){
				ano[cont] = linea.charAt(i);
				cont++;
			}else{
				cont = 0;
			}
			
			if (cont == 4){
				if ((i == linea.length()-1) || !Character.isDigit(linea.charAt(i+1))){
					encontrado = true;
					String bu = new String(ano);
					return bu;
				}else{
					cont = 0;
				}
			}
		}
		return null;
	}

	/**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    public static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("Seleccion de Archivos");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Add content to the window.
        frame.add(new Explorador());

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }
    
    
    /**
     * Genera la ventana con el <i>Explorador</i>
     * */
    public static void main(String[] args) {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                //Turn off metal's use of bold fonts
                UIManager.put("swing.boldMetal", Boolean.FALSE); 
                createAndShowGUI();
            }
        });
    }


}
