package lectura_ficheros;	//Ficheros del INE

import java.io.File;
import java.io.IOException;
enum FileType {xls, csv, txt, error}

/**
 * Mantiene y trata informacin sobre los ficheros que contienen datos con los que
 * GoodWave puede trabajar
 * */
public class FicheroDatos {

	private String rutaArchivo;
	private String nombreArchivo;
	private FileType extensionArchivo;	//sin '.'
	
	//ContenidoFichero contenido;
	String[] varUdsFecha;	//Vector que incluye {Variable de estudio, Unidades, Fecha}
	Tabla tabla;			//Tabla con los datos
	
	/**
	 * Constructor
	 * */
	public FicheroDatos(){
		rutaArchivo = null;
		nombreArchivo = null;
		extensionArchivo = null;	//sin '.'
		varUdsFecha = null;
		tabla = null;
		
	}
			
	/**
	 * Constructor por parametros
	 * @param fichero Fichero con los datos
	 * @param separador Caracter separador de los datos del fichero
	 */
	public FicheroDatos(File fichero, char separador){
		rutaArchivo = fichero.getAbsolutePath();
		nombreArchivo = fichero.getName();
		extensionArchivo = convertirTipoArchivoAEnum (nombreArchivo.substring(nombreArchivo.lastIndexOf(".")+1));
		varUdsFecha = null;
		tabla = null;
	} 
	
	/**
	 * Convierte una extensin de fichero al tipo <i>FileType</i>
	 * @param extension Fichero con los datos
	 * @return tipo enum correspondiente a <i>extension</i>
	 */
	private FileType convertirTipoArchivoAEnum (String extension){
		if (extension.equals("xls"))
			return FileType.xls;
		else if (extension.equals("csv"))
			return FileType.csv;
		else if (extension.equals("txt"))
			return FileType.txt;
		else
			return FileType.error;
		
		
	}

	/**
	 * Almacena en las variables de instancia los datos obtenidos del fichero <i>fichero</i>
	 * @param fichero Nombre del fichero a estudiar
	 * @param separador Caracter que separa los datos del fichero
	 * @param num_linea_objeto_de_estudio Numero de la linea que contiene la(s) variable(s) de estudio
	 * @param num_linea_unidades Numero de la linea que contiene las unidades
	 * @param num_linea_fecha Numero de la linea que contiene la(s) fecha(s) de los datos
	 * @param num_linea_comienzo_datos	Numero de la primera linea de datos del fichero 
	*/
	public void leerFicheroDatos(String fichero, char separador, int num_linea_objeto_de_estudio, int num_linea_unidades, int num_linea_fecha, int num_linea_comienzo_datos) throws IOException{
		switch (extensionArchivo){
			
		case xls:	//fichero tabular
			FicheroTabular ft = new FicheroTabular (fichero, separador, num_linea_objeto_de_estudio, num_linea_unidades, num_linea_fecha, num_linea_comienzo_datos);
			try {
					varUdsFecha = ft.getVarUdsFechaFTabular();
					tabla = ft.getDatos();
					
				} catch (IOException e) {
					e.printStackTrace();
				}	
			break;
			
		case csv:
			//TODO leer fichero
			break;
			
		case txt:
			//TODO leer fichero
			break;
			
		default:
			//error
			//TODO manejar error
		}		
	}
	
	/**
	 * Devuelve el parametro <i>varUdsFecha</i>
	 * @return <b>String</b> de la forma [variable_estudio, unids, fecha]
	*/
	public String[] getVarUdsFechaFTabular() throws IOException{
		return varUdsFecha;
	}
	
	/**
	 * Devuelve la variable de instancia <i>tabla</i>
	 * @return Tabla con los datos del fichero
	*/
	public Tabla getTabla() throws IOException{
		return tabla;
	}
}
