package lectura_ficheros;
import global.GlobalData;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.lang.String;
import java.util.StringTokenizer;

public class FicheroTabular {
	
	public Tabla tabla;
	public int numMuestras;
	public int numCols;
	
	String fichero;
	char separador;
	int num_linea_objeto_de_estudio;
	int num_linea_unidades;
	int num_linea_fecha;
	int num_linea_comienzo_datos;

	
	/**
	 * Constructor
	 * */
	public FicheroTabular (){
		tabla = null;
		numMuestras = numCols = 0;
		fichero = null;
		separador = '\t';
		num_linea_objeto_de_estudio = num_linea_unidades =	num_linea_fecha = num_linea_comienzo_datos = 0;
		
	}
	
	/**
	 * Constructor por parametros
	 * @param fichero Nombre del fichero a estudiar
	 * @param num_linea_objeto_de_estudio Numero de la linea que contiene la(s) variable(s) de estudio
	 * @param num_linea_unidades Numero de la linea que contiene las unidades
	 * @param num_linea_fecha Numero de la linea que contiene la(s) fecha(s) de los datos
	 * @param num_linea_comienzo_datos	Numero de la primera linea de datos del fichero 
	*/
	public FicheroTabular (String afichero, int anum_linea_objeto_de_estudio, int anum_linea_unidades, int anum_linea_fecha, int anum_linea_comienzo_datos){
		tabla = null;
		numMuestras = numCols = 0;
		fichero = afichero;
		separador = '\t';
		num_linea_objeto_de_estudio = anum_linea_objeto_de_estudio;
		num_linea_unidades = anum_linea_unidades;
		num_linea_fecha = anum_linea_fecha;
		num_linea_comienzo_datos = anum_linea_comienzo_datos;
	}
	
	/**
	 * Constructor por parametros
	 * @param fichero Nombre del fichero a estudiar
	 * @param separador Caracter que separa los datos del fichero
	 * @param num_linea_objeto_de_estudio Numero de la linea que contiene la(s) variable(s) de estudio
	 * @param num_linea_unidades Numero de la linea que contiene las unidades
	 * @param num_linea_fecha Numero de la linea que contiene la(s) fecha(s) de los datos
	 * @param num_linea_comienzo_datos	Numero de la primera linea de datos del fichero 
	*/
	public FicheroTabular (String afichero, char aseparador, int anum_linea_objeto_de_estudio, int anum_linea_unidades, int anum_linea_fecha, int anum_linea_comienzo_datos){
		tabla = null;
		numMuestras = numCols = 0;
		fichero = afichero;
		separador = aseparador;
		num_linea_objeto_de_estudio = anum_linea_objeto_de_estudio;
		num_linea_unidades = anum_linea_unidades;
		num_linea_fecha = anum_linea_fecha;
		num_linea_comienzo_datos = anum_linea_comienzo_datos;
	}

	/**
	 * Devuelve los datos "variable", "unidades" y "fecha" del fichero <i>fichero</i>
	 * @return vector de <b>String</b> de la forma [variable_estudio, unids, fecha]
	*/
	public String[] getVarUdsFechaFTabular() throws IOException{

		FileReader fr = null;
		BufferedReader entrada = null;
		String linea_aux=null, obj_estudio=null, unids=null, data=null;
		String[] metadatos = null;
		
		try {
			fr = new FileReader(fichero);
			entrada = new BufferedReader(fr);
			int cont = 0;
			int maximo = seleccionaMaximoDe3(num_linea_objeto_de_estudio, num_linea_unidades, num_linea_fecha);
			
			//bucle donde organizaremos los metadatos de las tablas
			//objeto de estudio, unidades y fecha
			do {
				cont++;
				linea_aux = entrada.readLine();
				if (cont == num_linea_objeto_de_estudio){
					obj_estudio = tratarLineaVariable(linea_aux);
				}
				if (cont == num_linea_unidades){
					if (num_linea_unidades == 0)
						unids = "Unidades";
					else
						unids = tratarLineaUds(linea_aux);
				}
				if (cont == num_linea_fecha){
					data = tratarLineaFecha(linea_aux);
				}
				
			} while ((linea_aux != null) && (cont != maximo));
			
			//PanelOpciones po= new PanelOpciones("Modificacion de datos", obj_estudio,unids,data,data);

			//guardamos los datos en el vector de String
			metadatos = new String[4];
			metadatos[0] = obj_estudio;
			metadatos[1] = unids;
			metadatos[2] = data;

		
		}finally{
			try{                    
				if( null != fr ){   
					fr.close();     
				}                  
			}catch (Exception e2){ 
				e2.printStackTrace();
			}
		}
		return metadatos;
	}

	/**
	 * Devuelve las fechas contenidas en una linea separadas por ";"
	 * @param linea_aux <b>String</b> las fechas separadas por el <i>separador</i>
	 * @return <b>String</b> con las fechas separadas por ";" 
	 * */
	private String tratarLineaFecha(String linea_aux) {
		//caso: varias columnas con diferentes fechas
		StringTokenizer st = new StringTokenizer(linea_aux, Character.toString(separador));
		String linea_procesada = new String(st.nextToken());
		int numTokens = st.countTokens();
		if (numTokens > 1){
			for (int i=0; i<numTokens; i++){
				linea_procesada = (linea_procesada + ";" + st.nextToken());
			}
		}
		return linea_procesada;
	}
	
	/**
	 * Devuelve la unidad de medida de los datos del fichero contenida en una linea
	 * @param linea_aux <b>String</b> la unidad de medida de los datos
	 * @return <b>String</b> con la unidad de medida  
	 * */
	private String tratarLineaUds(String linea_aux) {
		StringTokenizer st = new StringTokenizer(linea_aux, ":");
		int numTokens = st.countTokens();
		
		String linea_procesada = new String(st.nextToken());

		if (numTokens>1 && linea_procesada.compareToIgnoreCase("unidades") == 0){
			int pos = linea_aux.indexOf(":");
			linea_procesada = linea_aux.substring(pos+1);
		}
		return linea_procesada;
	}

	/**
	 * Devuelve las variables de estudio contenidas en una linea separadas por ";"
	 * @param linea_aux <b>String</b> las variables de estudio separadas por el <i>separador</i>
	 * @return <b>String</b> con las variables de estudio separadas por ";" 
	 * */
	private String tratarLineaVariable(String linea_aux) {
		StringTokenizer st = new StringTokenizer(linea_aux, Character.toString(separador));
		String linea_procesada = new String(st.nextToken());
		int numTokens = st.countTokens();
		if (numTokens > 1){
			for (int i=0; i<numTokens; i++){
				linea_procesada = (linea_procesada + ";" + st.nextToken());
			}
		}
		
		return linea_procesada;
	}

	/**
	 * Selecciona el maximo de 2 valores enteros
	 * @param uno Primer valor
	 * @param dos Segundo valor
	 * @param tres Tercer valor
	 * */
	private int seleccionaMaximoDe3(int uno, int dos, int tres) {
		int vmax = uno;
		if (dos > vmax)
			vmax = dos;
		if (tres > vmax)
			vmax = tres;
		return vmax;
	}


	//rescata la tabla en una estructura ContenidoFichero
	//y despues de eso ignora el resto del fichero
	/**
	 * Devuelve la tabla datos <i>tabla</i>
	 * @return Objeto <b>Tabla</b> con los datos de los lugares del fichero
	*/
	public Tabla getDatos() throws IOException{
		FileReader fr = null;
		BufferedReader entrada = null;
		StringTokenizer st = null;
		String linea_aux = null;
		int cont = 0;
		
		
		try {
			fr = new FileReader(fichero);
			entrada = new BufferedReader(fr);
			for (int i=0; i<num_linea_comienzo_datos;i++){
				 linea_aux = entrada.readLine();
			}
			st = new StringTokenizer (linea_aux, Character.toString(separador));
			numCols = st.countTokens();
			boolean fin_tabla = false;
			tabla = new Tabla(GlobalData.NUMFILAS, numCols);
			
			do {
				st = new StringTokenizer (linea_aux, Character.toString(separador));
				//condicion para descartar lineas sin datos del final del fichero
				if (st.countTokens() != numCols){
					fin_tabla=true;
				}else{
					tabla.insertaFila(st);
				}
				cont++;
				
			}while ((linea_aux=entrada.readLine()) != null && !fin_tabla);
			
			numMuestras = cont;
			
		}finally{
			try{                    
				if( null != fr ){   
					fr.close();     
				}                  
			}catch (Exception e2){ 
				e2.printStackTrace();
			}
		}
		
		//ajustamos la tabla a su tamao correcto
		Tabla tab2 = new Tabla(tabla.cursor, numCols);
		for (int i=0; i<tabla.cursor; i++)
		{
			tab2.tabla[i] = tabla.tabla[i];
		}
		tabla = tab2;
		return tabla;
	}
}

