package lectura_ficheros;

import java.lang.String;
import java.util.StringTokenizer;
import global.GlobalData;

/**
 * Clase que va a contener una tabla con filas del formato:
 * 
 * 		LUGAR	DATO	DATO	DATO	...
 * */
public class Tabla {
	
	public String [][] tabla;;
	public int numFilas;
	public int numCols;
	public int cursor;
	
	
	/**
	 * Constructor
	 * */
	public Tabla(){
		numFilas = 0;
		numCols = 0;
		cursor = 0;
		tabla = new String[1][1];
	}
	
	/**
	 * Constructor por parametros
	 * @param nfilas Numero de filas de la tabla
	 * @param ncols Numero de columnas de la tabla
	*/
	public Tabla(int nfilas, int ncols){
				numFilas = nfilas;
				numCols = ncols;
				cursor = 0;
				tabla = new String[nfilas][ncols];
	}
	
	/**
	 * Constructor por parametros
	 * Se establece el numero de filas al valor de una variable global
	 * @param ncols Numero de columnas de la tabla
	*/
	public Tabla(int ncols){
				numFilas = GlobalData.NUMFILAS;
				numCols = ncols;
				cursor = 0;
				tabla = new String[GlobalData.NUMFILAS][ncols];
	}

	/**
	 * Devuelve el numero de filas de la tabla
	 * @return Numero de filas de la tabla
	*/
	public int countFilas(){
		return numFilas;
	}
	
	/**
	 * Devuelve el numero de columnas de la tabla
	 * @return Numero de columnas de la tabla
	*/
	public int countCols(){
		return numCols;
	}

	/**
	 * Inserta una fila con datos en <i>tabla</i>
	 * @param st <b>StringTokenizer</b> con los datos de la fila a introducir
	 * */
	public void insertaFila(StringTokenizer st) {
		//Lo primero es el nombre, lo depuramos eliminando posibles cdigos numericos 
		tabla[cursor][0] = depurarNombreLugar(st.nextToken());
		for (int i=1; i<numCols; i++){
			String aux = st.nextToken();
			tabla[cursor][i] = aux;
		}
		cursor++;

		//comprobamos que no se ha alcanzado el mximo de espacio
		if (cursor == numFilas){
			Tabla tab2 = new Tabla(numFilas+GlobalData.NUMFILAS, numCols);
			for (int j=0; j<cursor; j++){
				tab2.tabla[j] = tabla[j];
				tab2.cursor++;
			}
			tabla = tab2.tabla;
			numFilas = tab2.numFilas;
		}
	}
	
	//devuelve un vector String con todos los nombres de lugares extrados de
	//nombreSinTratar, analizando caracteres los especiales "/", "(" y ")"
	/**
	 * Depura una linea con un nombre de lugar para eliminar codigos numericos
	 * que pueden dificultar la lectura
	 * @param nombreSinTratar <b>String</b> con la cadena sin tratar
	 * @return Objeto <b>String</b> con el nombre del lugar sin digitos
	 * */
	public String depurarNombreLugar(String nombreSinTratar) {
		String nombre = new String("");
		for (int i=0; i<nombreSinTratar.length(); i++){
			if (!Character.isDigit(nombreSinTratar.charAt(i))){
				nombre = nombre + nombreSinTratar.charAt(i);
			}
		}
		return nombre;
	}
}
